﻿using System;
using System.Threading.Tasks;
using System.Net.Http;
using System.Net;
using XServerClient;
using System.Globalization;
using XServerClient.Models;

namespace XServerJson
{
	class Program
	{
		static void Main()
		{
			ExecAsync().Wait();

			Console.ReadKey();
		}

		static async Task ExecAsync()
		{
			// initialize client for xserver-internet
			var xServer = new XServer(
			new Uri("https://xserver2-europe-eu-test.cloud.ptvgroup.com/"), new HttpClientHandler
			{
				Credentials = new NetworkCredential
				{
					Password = "<your xserver-internet token>",
					UserName = "xtok",
				}
			});

			try
			{
				var job = await xServer.StartPlanToursAsync(planToursRequestWithDirectDistance);

				var id = job.Id;
				while (job.Status == JobStatus.QUEUING || job.Status == JobStatus.RUNNING || job.Status == JobStatus.STOPPING)
				{
					await Task.Delay(1000);

					job = await xServer.WatchJobAsync(new WatchRequest { Id = id });
				}

				var result = await xServer.FetchToursResponseAsync(new JobRequest { Id = id });

				Console.WriteLine($@"{result.Tours.Count} tour(s) calculated, driving time: {
					TimeSpan.FromSeconds(result.CostReport.DrivingTime)}");
			}
			catch (XServerErrorException ex) {
				Console.WriteLine($"Request failed with error: {ex.Response.Content}");
			}
			finally
			{
				xServer.Dispose();
			}
		}

		// a simple xTour request, see from Raw-Request-Runner
		public static readonly PlanToursRequest planToursRequestWithDirectDistance = new PlanToursRequest
		{
			Locations = new TourPlanningLocation[] {
				new DepotSite {
					Id = "0",
					RouteLocation = new OffRoadRouteLocation {
						OffRoadCoordinate = new Coordinate {
							X = 6.116724014282227,
							Y = 49.60381302176553
						}
					}
				},
				new CustomerSite {
					Id = "1",
					RouteLocation = new OffRoadRouteLocation {
						OffRoadCoordinate = new Coordinate {
							X = 6.115951538085938,
							Y = 49.627836766637756
						}
					}
				},
				new CustomerSite {
					Id = "2",
					RouteLocation = new OffRoadRouteLocation {
						OffRoadCoordinate = new Coordinate {
							X = 6.157665252685548,
							Y = 49.618161960384896
						}
					}
				},
				new CustomerSite {
					Id = "3",
					RouteLocation = new OffRoadRouteLocation {
						OffRoadCoordinate = new Coordinate {
							X = 6.151227951049805,
							Y = 49.59763834158701
						}
					},
					OpeningIntervals = new[] {
						new StartDurationInterval {
							Start = DateTime.Parse("2020-05-13T10:00:00+01:00"),
							Duration = 3600
						},
						new StartDurationInterval {
							Start = DateTime.Parse("2020-05-13T14:00:00+01:00"),
							Duration = 3600
						}
					},
				},
				new CustomerSite {
					Id = "4",
					RouteLocation = new OffRoadRouteLocation {
						OffRoadCoordinate = new Coordinate {
							X = 6.076641082763673,
							Y = 49.61471415139774
						}
					}
				},
				new CustomerSite {
					Id = "5",
					RouteLocation = new OffRoadRouteLocation {
						OffRoadCoordinate = new Coordinate {
							X = 6.095695495605469,
							Y = 49.591017770932
						}
					},
					OpeningIntervals = new[] {
						new StartDurationInterval {
							Start = DateTime.Parse("2020-05-13T10:00:00+01:00"),
							Duration = 2700
						},
					},

				}
			},
			Orders = new Order[] {
				new PickupDeliveryOrder {
					Id = "0",
					PickupLocationId = "0",
					DeliveryLocationId = "1",
					Quantities = new double ? [] {
						1
					}
				},
				new PickupDeliveryOrder {
					Id = "1",
					PickupLocationId = "2",
					DeliveryLocationId = "0",
					Quantities = new double ? [] {
						1
					}
				},
				new PickupDeliveryOrder {
					Id = "2",
					PickupLocationId = "3",
					DeliveryLocationId = "4",
					Quantities = new double ? [] {
						1
					}
				},
				new VisitOrder {
					Id = "3",
					LocationId = "5"
				}
			},
			Fleet = new Fleet
			{
				Vehicles = new[] {
					new Vehicle {
						Ids = new[] {
							"vehicle1"
						},
						MaximumQuantityScenarios = new[] {
							new MaximumQuantityScenario {
								Quantities = new double ? [] {
									3
								}
							}
						}
					}
				}
			},
			DistanceMode = new DirectDistance { }
		};
	}
}